# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..gtract import gtractInvertBSplineTransform


def test_gtractInvertBSplineTransform_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        ignore_exception=dict(
            deprecated='1.0.0',
            nohash=True,
            usedefault=True,
        ),
        inputReferenceVolume=dict(argstr='--inputReferenceVolume %s', ),
        inputTransform=dict(argstr='--inputTransform %s', ),
        landmarkDensity=dict(
            argstr='--landmarkDensity %s',
            sep=',',
        ),
        numberOfThreads=dict(argstr='--numberOfThreads %d', ),
        outputTransform=dict(
            argstr='--outputTransform %s',
            hash_files=False,
        ),
        terminal_output=dict(
            deprecated='1.0.0',
            nohash=True,
        ),
    )
    inputs = gtractInvertBSplineTransform.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_gtractInvertBSplineTransform_outputs():
    output_map = dict(outputTransform=dict(), )
    outputs = gtractInvertBSplineTransform.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
