# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..nx import AverageNetworks


def test_AverageNetworks_inputs():
    input_map = dict(
        group_id=dict(usedefault=True, ),
        ignore_exception=dict(
            deprecated='1.0.0',
            nohash=True,
            usedefault=True,
        ),
        in_files=dict(mandatory=True, ),
        out_gexf_groupavg=dict(),
        out_gpickled_groupavg=dict(),
        resolution_network_file=dict(),
    )
    inputs = AverageNetworks.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_AverageNetworks_outputs():
    output_map = dict(
        gexf_groupavg=dict(),
        gpickled_groupavg=dict(),
        matlab_groupavgs=dict(),
    )
    outputs = AverageNetworks.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
